"use strict";
this.name	= "ils";
this.author	= "Norby";
this.copyright	= "2014 Norby";
this.description= "Instrument Landing System";
this.version	= "1.2";
this.licence	= "CC BY-NC-SA 4.0";


//internal properties, should not touch
this.$ILSF = null; //FCB pointer
this.$ILSP = null; //Previous Heading in FCB
this.$ILSS = null; //target station

//worldscript events
//this.startUp = function() {
//}

this.equipmentDamaged = this.equipmentDestroyed = function(equipment) {
	if(equipment == "EQ_ILS") this.$ClearILS(); //stop ILS if working
}

this.shipTargetAcquired = function() {
	var p = player.ship;
	if(p && p.equipmentStatus("EQ_ILS") == "EQUIPMENT_OK") {
		var t = p.target;
		if( this.$ILSTarget( t ) ) {
			this.$ILSS = t; //target station
			if( !isValidFrameCallback( this.$ILSF ) )
				this.$ILSF = addFrameCallback( this.$FCB.bind(this) );
			player.consoleMessage("ILS set to "+t.displayName, 10);
//			log(this.name, "ILS ON to: "+t+" "+this.$ILSF);//debug
		} else this.$ClearILS();
	} else this.$ClearILS();
}

/* another oxp cause a bug which turn off ILS immediately in the next lines so do not use this event
this.shipTargetLost = function() {
	log(this.name, "ILS OFF 1");//debug
	this.$ClearILS();
}
*/

this.shipWillDockWithStation = this.shipWillEnterWitchspace = this.shipDied = function() {
	this.$ClearILS(true);//nomsg
}

//ILS methods
this.$ILSTarget = function(t) {
	if( t && t.isValid && t.isStation && t.dataKey.indexOf("carrier") == -1 )
		return true;
	else return false;
}

this.$ClearILS = function(nomsg) {
	this.$ILSS = null;
	this.$ILSP = null;
	if( this.$ILSV && this.$ILSV.isVisualEffect ) {
		this.$ILSV.remove();
		if(!nomsg) player.consoleMessage("ILS off");
	}
	this.$ILSV = null;
	if( isValidFrameCallback( this.$ILSF ) ) removeFrameCallback( this.$ILSF );
//	log(this.name, "ILS OFF");//debug
}

this.$FCB = function( delta ) {
//	var w = worldScripts.ils;
//	player.consoleMessage(this.$ILSS+" "+delta); //debug
	if( !this.$ILSS ) return;
	var p = player.ship;
	if( this.$ILSTarget( p.target ) ) {
		var ph = p.heading;
		if( !this.$ILSP ) this.$ILSP = ph;
		var pp = p.position;
		var s = this.$ILSS; //station
		var mp = p.maxPitch;
		var mr = p.maxRoll;
		var my = p.maxYaw;
//		log(this.name, "Station: "+s);//debug

		var u = s.subEntities;
		var i = 0;
		if( u ) while( u[i] && !u[i].isDock ) i++;
		var b = s;
		var bp = b.position;
		var dd = 0;
		if( u[i] && u[i].isDock ) {
			b = u[i];
			dd = b.position.magnitude(); //dock distance from the center of the station
			bp = s.position.add( s.heading.direction().multiply(dd) );
		}
		var d = pp.distanceTo( bp ) / 2;

		var w = s.position.subtract( pp ).direction();
		var an = s.heading.angleTo(w);
//		player.consoleMessage(d +" "+ s.heading.angleTo(w)); //debug
		if( an < 3.14 ) { //player is not before the station yet
			var na = 3.14-an;
			na = na * na;
			d += na * 2000;//extend distance if apporaching behind from the station
		}
		
		d = Math.min( 9500, d ); //max. at the Nav Buoy
		if(d < 10) this.$ClearILS(true); //for sure, do not start turn back if crossed the point in the dock
		var a = bp.add( s.heading.direction().multiply( d ) ); //aimed position

		var v = a.subtract( pp ).direction();
		
		
		
/*		if( d > 10000 ) { //far approach
			var a = s.position.add( s.heading.direction().multiply(5000) ); //aimed position
		} else { //near approach
			var u = s.subEntities;
			var i = 0;
			if( u ) while( u[i] && !u[i].isDock ) i++;
			var b = s;
			var bp = s.position;
			if( u[i] && u[i].isDock ) {
				b = u[i]; //to the dock subentity
				bp = bp.add( b.position ); //the subent pos is relative to the station's pos
			}
			d = pp.distanceTo( bp ); //refine the distance to the center of the dock
			//the aimed position will slowly approach the dock
//			log(this.name, "Dist: "+d+" Dock: "+b);//debug
			var a = bp.add( s.heading.direction().multiply( d / 2 ) );
		}
*/
		if( this.$ILSV && this.$ILSV.isVisualEffect )
			this.$ILSV.position = a;
		else this.$ILSV = system.addVisualEffect("ils-marker", a);
		
		var angle = ph.angleTo(v);

		var ato = 0;
		if(ph && this.$ILSP) ato = ph.angleTo(this.$ILSP);//angle to the Previous Heading
		if( ato < 0.005 && angle > 0.0015 ) { //steer if no manual steering and not in 0.1 degree
			//if the above ato value lower then can not start steering in my intel atom netbook
//			player.consoleMessage(this.$ILSP +" vs "+ ph); //debug
			var a = Math.min(p.maxPitch*delta, angle/12);//half max turn/step and not too accutate
/*			var wstow = worldScripts["towbar"];  //slowed steering from towbar oxp if there are towed ship
			if( wstow && wstow.$TowbarShip && wstow.$TowbarShip.isValid ) {
				var ma = Math.min( 2, ps.mass / wstow.$TowbarShip.mass ); ///small ship max. 2x
				a = a * ma / 3; // 1/3 of the original rate with same mass, min. 1/5 max. 2/3
//				player.consoleMessage("ILS slow steering with towed ship "+ato);//debug
			}
*/			var c = ph.cross(v).direction(); //set the plane where we should rotate in
			var q = p.orientation.rotate( c, -a );
			p.orientation = q;
		}
		
		this.$ILSP = p.heading;
	} else this.$ClearILS(); //no target or a new target is not station
}